import json
import subprocess

class FoldershareException(Exception):
    def __init__(self, output):
        self.output = output
        super().__init__(self.output)

class Foldershare:

    def __init__(self, foldershare_client_path = "./foldershare", url=None, host="localhost", port=80, username=None, password=None, authfile=None, masquerade=None, apikey=None):
        self.foldershare_client_path = foldershare_client_path
        self.base_command = [self.foldershare_client_path]

        self.authfile = authfile

        if not self.authfile:
            if url:
                host = url
                port = None
            else:
                if not host or not port:
                    raise ValueError("Please provide the 'host' & 'port' for the site, when not providing a url.")

            if host:
                self.base_command += ["--host", host]

            if port:
                self.base_command += ["--port", port]

            if masquerade or apikey:
                if not masquerade:
                    raise ValueError("Please provide a 'masquerade' username")
                if not apikey:
                    raise ValueError("Please provide an 'apikey' value")

                self.base_command += ["--masquerade", masquerade, "--apikey", apikey]
            elif username or password:
                if not username:
                    raise ValueError("Please provide a username")

                if not password:
                    raise ValueError("Please provide a password")

                self.base_command += ["--username", username, "--password", password]
        else:
            self.base_command += ["--authfile", self.authfile]

        self.base_command += ["--format", "json"]

    def __getattr__(self, item):
        command_name = item
        def command_func(*args):
            return self._run_command(self.base_command + [command_name,] + list(args))
        return command_func

    def _run_command(self, command):
        command = [str(arg) for arg in command]
        result = subprocess.run(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        std_output = result.stdout.decode('utf-8')
        error_output = result.stderr.decode("utf-8")
        if result.returncode != 0:
            # an error occurred
            raise FoldershareException(error_output)
        try:
            return json.loads(std_output)
        except:
            return std_output

    def help(self):
        return self._run_command(self.base_command + ["--help"])