import os

from foldershare_wrapper_api import Foldershare
from pathlib import Path
from pprint import pprint
import shutil

FOLDERSHARE_BINARY_PATH = "../foldershare"
FOLDERSHARE_URL = ""
FOLDERSHARE_USERNAME = ""
FOLDERSHARE_PASSWORD = ""

# The names of test folders and files to create and modify.
FLDR="DeletableFolder$$"
SUBFLDR="DeletableSubfolder$$"
FILE="DeletableFile$$.txt"

VERBOSE = False

def test_section(section):
    print (f"TEST: {section}")

def test_subsection(subsection):
    print (f"# {subsection}")

def test_spacing():
    print("\n")

def test_command(fs, command_string, silent=not VERBOSE):
    print (f"> {command_string}")
    args = command_string.split(" ")
    command_name = args.pop(0)

    result = getattr(fs, command_name)(*args)
    if not silent:
        pprint(result)
        print("\n")

if __name__ == "__main__":
    fs = Foldershare(foldershare_client_path=FOLDERSHARE_BINARY_PATH, url=FOLDERSHARE_URL, username=FOLDERSHARE_USERNAME, password=FOLDERSHARE_PASSWORD)

    # Show server version.
    test_command(fs, "version", silent=True)

    # Show server configuration
    test_command(fs, "config", silent=True)

    # Show server usage
    test_command(fs, "usage", silent=True)

    # Stat
    test_command(fs, f"rm -rf /{FLDR}", silent=True)
    test_command(fs, f"rm -rf /{FILE}", silent=True)

    Path(FILE).touch()

    ## Stat on root
    test_spacing()
    test_spacing()
    test_section("Stat on root")
    test_command(fs, f"stat /")
    test_command(fs, f"stat personal:/")
    test_command(fs, f"stat public:/")
    test_command(fs, f"stat personal://1/")
    test_command(fs, f"stat public://1/")
    test_command(fs, f"stat personal://admin/")
    test_command(fs, f"stat public://admin/")
    test_command(fs, f"stat -t /")
    test_command(fs, f"stat --terse /")

    test_spacing()
    test_subsection("Stat on folder")
    test_command(fs, f"mkdir /{FLDR}")
    test_command(fs, f"stat /{FLDR}")
    test_command(fs, f"stat personal:/{FLDR}")
    test_command(fs, f"stat -t /{FLDR}")
    test_command(fs, f"mkdir /{FLDR}/{SUBFLDR}")
    test_command(fs, f"stat /{FLDR}/{SUBFLDR}")
    test_command(fs, f"stat -t /{FLDR}/{SUBFLDR}")

    test_spacing()
    test_subsection("Stat on file")
    test_command(fs, f"upload -v {FILE} /")
    test_command(fs, f"stat /{FILE}")
    test_command(fs, f"stat -t /{FILE}")

    test_command(fs, f"upload -v {FILE} /{FLDR}")
    test_command(fs, f"stat /{FLDR}/{FILE}")
    test_command(fs, f"stat -t /{FLDR}/{FILE}")

    test_command(fs, f"upload -v {FILE} /{FLDR}/{SUBFLDR}")
    test_command(fs, f"stat /{FLDR}/{SUBFLDR}/{FILE}")
    test_command(fs, f"stat -t /{FLDR}/{SUBFLDR}/{FILE}")

    test_command(fs, f"rm -rf /{FLDR}/{SUBFLDR}/{FILE}", silent=True)
    test_command(fs, f"rm -rf /{FLDR}/{SUBFLDR}", silent=True)
    test_command(fs, f"rm -rf /{FLDR}/{FILE}", silent=True)
    test_command(fs, f"rm -rf /{FLDR}", silent=True)
    test_command(fs, f"rm -f /{FILE}", silent=True)
    os.remove(FILE)

    test_spacing()
    test_spacing()
    test_section("Ls")

    test_command(fs, f"rm -rf /{FLDR}")
    test_command(fs, f"rm -rf /{FILE}")
    Path(FILE).touch()

    test_command(fs, f"upload -v {FILE} /")

    test_spacing()
    test_subsection("Ls on root")
    test_command(fs, f"ls /")
    test_command(fs, f"ls -d /")
    test_command(fs, f"ls -F /")
    test_command(fs, f"ls -i /")
    test_command(fs, f"ls -l /")
    test_command(fs, f"ls -s /")
    test_command(fs, f"ls -S /")
    test_command(fs, f"ls -t /")
    test_command(fs, f"ls -silFS /")

    test_spacing()
    test_subsection("Ls on subfolder")
    test_command(fs, f"mkdir /{FLDR}")
    test_command(fs, f"ls -l /{FLDR}")
    test_command(fs, f"mkdir /{FLDR}/{SUBFLDR}")
    test_command(fs, f"ls -l /{FLDR}/{SUBFLDR}")

    test_spacing()
    test_subsection("Ls on file")
    test_command(fs, f"ls -l /{FILE}")

    test_command(fs, f"upload -v {FILE} /{FLDR}")
    test_command(fs, f"ls -l /{FLDR}/{FILE}")
    test_command(fs, f"upload -v {FILE} /{FLDR}/{SUBFLDR}")
    test_command(fs, f"ls -l /{FLDR}/{SUBFLDR}/{FILE}")

    test_spacing()
    test_subsection("Ls recursive")
    test_command(fs, "ls -R /")

    test_command(fs, f"rm -rf /{FLDR}/{SUBFLDR}/{FILE}", silent=True)
    test_command(fs, f"rm -rf /{FLDR}/{SUBFLDR}", silent=True)
    test_command(fs, f"rm -rf /{FLDR}/{FILE}", silent=True)
    test_command(fs, f"rm -rf /{FLDR}", silent=True)
    test_command(fs, f"rm -f /{FILE}", silent=True)
    os.remove(FILE)

    test_spacing()
    test_spacing()
    test_section("Download")
    test_command(fs, f"mkdir /{FLDR}")
    test_command(fs, f"mkdir /{FLDR}/{SUBFLDR}")
    Path(FILE).touch()
    test_command(fs, f"upload {FILE} /", silent=True)
    test_command(fs, f"upload {FILE} /{FLDR}", silent=True)
    test_command(fs, f"upload {FILE} /{FLDR}/{SUBFLDR}", silent=True)
    os.remove(FILE)

    test_spacing()
    test_subsection("Download file")

    # To default current directory.
    test_command(fs, f"download -v /{FILE}")
    if not os.path.exists(FILE):
        raise Exception("Error: file did not download")
    os.remove(FILE)


    # To explicit current directory.
    test_command(fs, f"download -v /{FILE} .")
    if not os.path.exists(FILE):
        raise Exception("Error: file did not download")
    os.remove(FILE)

    # To explicit new file in current directory.
    test_command(fs, f"download -v /{FILE} {FILE}")
    if not os.path.exists(FILE):
        raise Exception("Error: file did not download")
    os.remove(FILE)

    # To default current directory.
    test_command(fs, f"download -v /{FLDR}/{FILE}")
    if not os.path.exists(FILE):
        raise Exception("Error: file did not download")
    os.remove(FILE)

    test_command(fs, f"download -v /{FLDR}/{SUBFLDR}/{FILE}")
    if not os.path.exists(FILE):
        raise Exception("Error: file did not download")
    os.remove(FILE)

    test_spacing()
    test_subsection("Download folder")
    test_command(fs, f"download -v /{FLDR}/{SUBFLDR}")
    if not os.path.exists(SUBFLDR):
        raise Exception("Error: folder did not download")
    shutil.rmtree(SUBFLDR)

    test_command(fs, f"download -v /{FLDR}")
    if not os.path.exists(FLDR):
        raise Exception("Error: folder did not download")
    shutil.rmtree(FLDR)


    # To explicit new file in current directory.
    test_command(fs, f"download -v /{FLDR} {FLDR}")
    if not os.path.exists(FLDR):
        raise Exception("Error: folder did not download")
    shutil.rmtree(FLDR)

    test_command(fs, f"rm -rf /{FLDR}/{SUBFLDR}/{FILE}", silent=True)
    test_command(fs, f"rm -rf /{FLDR}/{SUBFLDR}", silent=True)
    test_command(fs, f"rm -rf /{FLDR}/{FILE}", silent=True)
    test_command(fs, f"rm -rf /{FLDR}", silent=True)
    test_command(fs, f"rm -f /{FILE}", silent=True)

    #------------------------------------------------------------------------
    # POST operations.
    #------------------------------------------------------------------------

    # Make folders
    FLDR1=f"{FLDR}_1"
    FLDR2=f"{FLDR}_2"
    FLDR3=f"{FLDR}_3"
    SUBFLDR1=f"{SUBFLDR}_1"
    SUBFLDR2=f"{SUBFLDR}_2"

    test_spacing()
    test_spacing()
    test_section("Mkdir")

    test_command(fs, f"rm -rf /{FLDR} /{FLDR1} /{FLDR2} /{FLDR3}", silent=True)
    test_command(fs, f"mkdir -v /{FLDR}")
    test_command(fs, f"mkdir -v /{FLDR1} /{FLDR2} /{FLDR3}")
    test_command(fs, f"ls -l /")

    test_command(fs, f"mkdir -v /{FLDR}/{SUBFLDR}")
    test_command(fs, f"ls -l /{FLDR}")

    test_command(fs, f"mkdir -v /{FLDR1}/{SUBFLDR1} /{FLDR1}/{SUBFLDR2}")
    test_command(fs, f"ls -l /{FLDR1}")

    test_command(fs, f"rm -rf /{FLDR} /{FLDR1} /{FLDR2} /{FLDR3}", silent=True)

    test_command(fs, f"mkdir -v -p /{FLDR1}/{SUBFLDR1}/{SUBFLDR2} /{FLDR1}/{SUBFLDR2}/{SUBFLDR1}")
    test_command(fs, f"ls -lR /{FLDR1}")
    test_command(fs, f"rm -rf /{FLDR1}", silent=True)

    # Upload files
    LOCAL_FOLDER=f"{FLDR}_Local"
    REMOTE_FOLDER=f"{FLDR}_Remote"

    test_spacing()
    test_spacing()
    test_section("Upload")

    test_command(fs, f"rm -rf /{FLDR} /{LOCAL_FOLDER} /{REMOTE_FOLDER}", silent=True)
    test_command(fs, f"rm -f /{FILE}", silent=True)
    test_command(fs, f"mkdir /{FLDR} /{FLDR}/{SUBFLDR}")
    Path(FILE).touch()
    test_command(fs, f"mkdir -p {LOCAL_FOLDER} {LOCAL_FOLDER}/sub1 {LOCAL_FOLDER}/sub1/sub2")

    os.makedirs(f"{LOCAL_FOLDER}/sub1/sub2", exist_ok=True)
    Path(f"{LOCAL_FOLDER}/file1.txt").touch()
    Path(f"{LOCAL_FOLDER}/file2.txt").touch()
    Path(f"{LOCAL_FOLDER}/sub1/file3.txt").touch()
    Path(f"{LOCAL_FOLDER}/sub1/file4.txt").touch()
    Path(f"{LOCAL_FOLDER}/sub1/sub2/file5.txt").touch()

    test_spacing()
    test_subsection("Upload file")
    test_command(fs, f"upload -v {FILE} /")
    test_command(fs, f"upload -v {FILE} /{FLDR}")
    test_command(fs, f"upload -v {FILE} /{FLDR}/{SUBFLDR}")

    test_spacing()
    test_subsection("Upload folder")
    test_command(fs, f"upload -v {LOCAL_FOLDER} /")
    test_command(fs, f"mkdir /{REMOTE_FOLDER}")
    test_command(fs, f"upload -v {LOCAL_FOLDER} /{REMOTE_FOLDER}")

    test_command(fs, f"rm -rf /{FLDR}/{SUBFLDR}/{FILE}", silent=True)
    test_command(fs, f"rm -rf /{FLDR}/{SUBFLDR}", silent=True)
    test_command(fs, f"rm -rf /{FLDR}/{FILE}", silent=True)
    test_command(fs, f"rm -rf /{FLDR}", silent=True)
    test_command(fs, f"rm -f /{FILE}", silent=True)
    test_command(fs, f"rm -rf /{LOCAL_FOLDER}", silent=True)
    test_command(fs, f"rm -f /{REMOTE_FOLDER}", silent=True)

    os.remove(FILE)
    shutil.rmtree(LOCAL_FOLDER)

    #------------------------------------------------------------------------
    # DELETE operations.
    #------------------------------------------------------------------------

    # Remove empty folders
    SUBFLDR1=f"{SUBFLDR}_1"
    SUBFLDR2=f"{SUBFLDR}_2"

    test_spacing()
    test_spacing()
    test_section("Rmdir")
    test_command(fs, f"rm -rf /{FLDR}", silent=True)
    test_command(fs, f"mkdir -v /{FLDR} /{FLDR}/{SUBFLDR1} /{FLDR}/{SUBFLDR2}")

    test_command(fs, f"rmdir -v /{FLDR}/{SUBFLDR2}", silent=True)
    test_command(fs, f"rmdir -v /{FLDR}/{SUBFLDR1}", silent=True)
    test_command(fs, f"rmdir -v /{FLDR}", silent=True)

    # Remove
    SUBFLDR1=f"{SUBFLDR}_1"
    SUBFLDR2=f"{SUBFLDR}_2"

    test_spacing()
    test_spacing()
    test_section("Rm")

    test_command(fs, f"mkdir -v /{FLDR} /{FLDR}/{SUBFLDR1} /{FLDR}/{SUBFLDR2}")
    Path(FILE).touch()
    test_command(fs, f"upload {FILE} /", silent=True)
    test_command(fs, f"upload {FILE} /{FLDR}", silent=True)
    test_command(fs, f"upload {FILE} /{FLDR}/{SUBFLDR1}", silent=True)
    os.remove(FILE)

    test_spacing()
    test_subsection("rm empty folder")
    test_command(fs, f"rm -d /{FLDR}/{SUBFLDR2}")

    test_spacing()
    test_subsection("rm non-empty folder (recursive)")
    test_command(fs, f"rm -rf /{FLDR}/{SUBFLDR1}")
    test_command(fs, f"rm -rf /{FLDR}")

    test_spacing()
    test_subsection("rm file")
    test_command(fs, f"rm /{FILE}")

    #------------------------------------------------------------------------
    # PATCH operations.
    #------------------------------------------------------------------------

    # Copy
    COPYFLDR=f"Copy_{FLDR}"
    COPYFILE=f"Copy_{FILE}"
    COPYFILE2=f"Copy2_{FILE}"

    test_spacing()
    test_spacing()
    test_section("Cp")

    test_command(fs, f"rm -rf /{FLDR}", silent=True)
    test_command(fs, f"mkdir /{FLDR}", silent=True)
    Path(FILE).touch()
    test_command(fs, f"rm -f /{FILE}", silent=True)
    test_command(fs, f"upload {FILE} /", silent=True)
    test_command(fs, f"upload {FILE} /{FLDR}", silent=True)

    os.remove(FILE)

    test_spacing()
    test_subsection("cp file")
    test_command(fs, f"cp -v /{FILE} /{COPYFILE}")
    test_command(fs, f"cp -v /{FILE} /{FLDR}")
    test_command(fs, f"cp -v /{FILE} /{FLDR}/{COPYFILE}")
    test_command(fs, f"cp -v /{FLDR}/{COPYFILE} /{COPYFILE2}")

    test_spacing()
    test_subsection("cp folder")
    test_command(fs, f"cp -v /{FLDR} /{COPYFLDR}")
    test_command(fs, f"cp -v /{FLDR} /{COPYFLDR}")

    # Cleanup.
    test_command(fs, f"rm -rf /{FLDR} /{COPYFLDR} /{FILE} /{COPYFILE} /{COPYFILE2}", silent=True)

    # Move
    RENAMEDFLDR=f"Renamed_{FLDR}"
    RENAMEDSUBFLDR=f"Renamed_{SUBFLDR}"
    RENAMEDFILE=f"Renamed_{FILE}"

    test_spacing()
    test_spacing()
    test_section("Mv")
    test_command(fs, f"rm -rf /{FLDR}", silent=True)
    test_command(fs, f"rm -rf /{RENAMEDFLDR} /{RENAMEDFILE}", silent=True)
    test_command(fs, f"mkdir /{FLDR} /{FLDR}/{SUBFLDR}", silent=True)
    Path(FILE).touch()
    test_command(fs, f"rm -f /{FILE}", silent=True)
    test_command(fs, f"upload {FILE} /", silent=True)
    test_command(fs, f"upload {FILE} /{FLDR}", silent=True)
    os.remove(FILE)

    test_spacing()
    test_subsection("mv to rename file")
    test_command(fs, f"mv -v --wait /{FILE} /{RENAMEDFILE}")
    test_command(fs, f"mv -v --wait /{FLDR}/{FILE} /{FLDR}/{RENAMEDFILE}")

    test_spacing()
    test_subsection("mv to rename folder")
    test_command(fs, f"mv -v --wait /{FLDR}/{SUBFLDR} /{FLDR}/{RENAMEDSUBFLDR}")
    test_command(fs, f"mv -v --wait /{FLDR} /{RENAMEDFLDR}")

    test_spacing()
    test_subsection("mv file to folder")
    test_command(fs, f"mv -v --wait /{RENAMEDFILE} /{RENAMEDFLDR}/{RENAMEDSUBFLDR}")

    test_spacing()
    test_subsection("mv file to root")
    test_command(fs, f"mv -v --wait /{RENAMEDFLDR}/{RENAMEDFILE} /")

    test_spacing()
    test_subsection("mv file to root")
    test_command(fs, f"mv -v /{RENAMEDFLDR}/{RENAMEDSUBFLDR} /")
    test_command(fs, f"wait -v /{RENAMEDSUBFLDR}")

    test_spacing()
    test_subsection("mv folder to subfolder")
    test_command(fs, f"mv -v --wait /{RENAMEDSUBFLDR} /{RENAMEDFLDR}/{SUBFLDR}")

    test_command(fs, f"rm -rf /{RENAMEDFLDR} /{RENAMEDFILE}", silent=True)

    # Update
    UPDATEDFLDR=f"Updated_{FLDR}"
    UPDATEDFILE=f"Updated_{FILE}"

    test_spacing()
    test_spacing()
    test_section("Update")

    test_command(fs, f"rm -rf /{FLDR}", silent=True)
    test_command(fs, f"mkdir /{FLDR}", silent=True)
    Path(FILE).touch()
    test_command(fs, f"rm -f /{FILE}", silent=True)
    test_command(fs, f"upload {FILE} /", silent=True)
    os.remove(FILE)

    test_spacing()
    test_subsection("update name")
    test_command(fs, f"update name {UPDATEDFILE} /{FILE}")
    test_command(fs, f"update name {UPDATEDFLDR} /{FLDR}")
    test_command(fs, f"ls -l /")

    test_spacing()
    test_subsection("update description")
    test_command(fs, f"update description 'description' /{UPDATEDFILE}")
    test_command(fs, f"update description 'description' /{UPDATEDFLDR}")
    test_command(fs, f"rm -rf /{UPDATEDFILE} /{UPDATEDFLDR}", silent=True)

    test_spacing()
    test_spacing()
    print("All tests are OK.")